@@ -125,3 +125,23 @@ gvfs_setup_debug_handler (void)
   sa.sa_flags = 0;
   sigaction (SIGUSR2, &sa, NULL);
 }
+
+gboolean
+gvfs_is_ipv6 (const char *host)
+{
+  const char *p = host;
+
+  g_return_val_if_fail (host != NULL, FALSE);
+
+  if (*p != '[')
+    return FALSE;
+
+  while (++p)
+    if (!g_ascii_isxdigit (*p) && *p != ':')
+      break;
+
+  if (*p != ']' || *(p + 1) != '\0')
+    return FALSE;
+
+  return TRUE;
+}
