@@ -624,7 +624,7 @@ try_login:
         {
           g_debug ("# error querying remote address: %s\nUsing original address instead.\n", task.error->message);
           g_vfs_ftp_task_clear_error (&task);
-          ftp->addr = g_object_ref (addr);
+          ftp->addr = G_SOCKET_CONNECTABLE (g_object_ref (addr));
         }
     }
 
@@ -775,7 +775,7 @@ error_550_permission_or_not_found (GVfsFtpTask *task, gpointer file)
       g_set_error_literal (&task->error,
                            G_IO_ERROR,
                            G_IO_ERROR_NOT_FOUND,
-                           _("File doesn't exist"));
+                           _("File doesn’t exist"));
     }
 }
 
@@ -1169,7 +1169,7 @@ do_query_info (GVfsBackend *backend,
       g_set_error_literal (&task.error,
                            G_IO_ERROR,
                            G_IO_ERROR_NOT_FOUND,
-                           _("File doesn't exist"));
+                           _("File doesn’t exist"));
     }
 
   g_vfs_ftp_task_done (&task);
@@ -1589,7 +1589,7 @@ do_pull_improve_error_message (GVfsFtpTask *task,
 	      if (file_type == G_FILE_TYPE_DIRECTORY)
 		{
 		  g_set_error_literal (&task->error, G_IO_ERROR, G_IO_ERROR_WOULD_MERGE,
-                                       _("Can't copy directory over directory"));
+                                       _("Can’t copy directory over directory"));
 		  return;
 		}
 	      /* continue to would_recurse error */
@@ -1612,7 +1612,7 @@ do_pull_improve_error_message (GVfsFtpTask *task,
 	}
       
       g_set_error_literal (&task->error, G_IO_ERROR, G_IO_ERROR_WOULD_RECURSE,
-                           _("Can't recursively copy directory"));
+                           _("Can’t recursively copy directory"));
     }
 }
 
@@ -1741,6 +1741,7 @@ try_query_fs_info (GVfsBackend *backend,
                    GFileAttributeMatcher *matcher)
 {
   g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE, "ftp");
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE, TRUE);
   g_vfs_job_succeeded (G_VFS_JOB (job));
   return TRUE;
 }
