@@ -100,7 +100,7 @@ http_path_get_basename (const char *path)
     return NULL;
 
   /* remove any leading slashes */
-  while (*path != '\0' && (*path == '/' || *path == ' '))
+  while (*path != '\0' && *path == '/')
     path++;
 
   len = strlen (path);
@@ -111,7 +111,7 @@ http_path_get_basename (const char *path)
   while (len)
     {
       char c = path[len - 1];
-      if (!g_ascii_isspace (c) && c != '/')
+      if (c != '/')
 	break;
 
       len--;
@@ -682,6 +682,7 @@ try_query_fs_info (GVfsBackend *backend,
                    GFileAttributeMatcher *matcher)
 {
   g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE, "http");
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE, TRUE);
   g_vfs_job_succeeded (G_VFS_JOB (job));
   return TRUE;
 }
