@@ -90,7 +90,6 @@ struct _GVfsBackendSmbBrowse
   GMutex update_cache_lock;
   time_t last_entry_update;
   GList *entries;
-  int entry_errno;
 };
 
 
@@ -548,7 +547,6 @@ update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
   char dirents[1024*4];
   struct smbc_dirent *dirp;
   GList *entries;
-  int entry_errno;
   SMBCFILE *dir;
   int res;
   smbc_opendir_fn smbc_opendir;
@@ -557,7 +555,6 @@ update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
 
 
   entries = NULL;
-  entry_errno = 0;
   res = -1;
 
   g_mutex_lock (&backend->update_cache_lock);
@@ -575,7 +572,6 @@ update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
   g_free (uri);
   if (dir == NULL)
     {
-      entry_errno = errno;
       goto out;
     }
 
@@ -632,7 +628,6 @@ update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
   /* Clear old cache */
   g_list_free_full (backend->entries, (GDestroyNotify)browse_entry_free);
   backend->entries = entries;
-  backend->entry_errno = entry_errno;
   backend->last_entry_update = time (NULL);
 
   g_debug ("update_cache - done.\n");
@@ -792,6 +787,7 @@ do_mount (GVfsBackend *backend,
   GMountSpec *browse_mount_spec;
   smbc_opendir_fn smbc_opendir;
   smbc_closedir_fn smbc_closedir;
+  int errsv;
   
   smb_context = smbc_new_context ();
   if (smb_context == NULL)
@@ -903,6 +899,8 @@ do_mount (GVfsBackend *backend,
 
   g_debug ("do_mount - URI = %s\n", uri);
 
+  errsv = 0;
+
   do
     {
       op_backend->mount_try_again = FALSE;
@@ -912,12 +910,13 @@ do_mount (GVfsBackend *backend,
 
       dir = smbc_opendir (smb_context, uri);
 
+      errsv = errno;
       g_debug ("do_mount - [%s; %d] dir = %p, cancelled = %d, errno = [%d] '%s' \n",
              uri, op_backend->mount_try, dir, op_backend->mount_cancelled,
-             errno, g_strerror (errno));
+             errsv, g_strerror (errsv));
 
       if (dir == NULL && 
-          (op_backend->mount_cancelled || (errno != EPERM && errno != EACCES)))
+          (op_backend->mount_cancelled || (errsv != EPERM && errsv != EACCES)))
         {
           g_debug ("do_mount - (errno != EPERM && errno != EACCES), cancelled = %d, breaking\n", op_backend->mount_cancelled);
 	  break;
@@ -965,7 +964,7 @@ do_mount (GVfsBackend *backend,
         g_vfs_job_failed (G_VFS_JOB (job),
 			  G_IO_ERROR, G_IO_ERROR_FAILED,
 			  /* translators: We tried to mount a windows (samba) share, but failed */
-			  _("Failed to retrieve share list from server: %s"), g_strerror (errno));
+			  _("Failed to retrieve share list from server: %s"), g_strerror (errsv));
 
       return;
     }
