@@ -96,7 +96,9 @@ send_reply (GVfsJob *job)
 {
   GVfsJobSeekRead *op_job = G_VFS_JOB_SEEK_READ (job);
   
-  g_debug ("job_seek_read send reply, pos %d\n", (int)op_job->final_offset);
+  g_debug ("send_reply(%p), pos=%"G_GOFFSET_FORMAT", failed=%d (%s)\n",
+           job, op_job->final_offset,
+           job->failed, job->failed ? job->error->message : "");
 
   if (job->failed)
     g_vfs_channel_send_error (G_VFS_CHANNEL (op_job->channel), job->error);
